

/*
	PA0	O	A
	PA1	O	C2
	PA2	O	B
	PA3	O	F
	PA4	O	C4
	PA5	O	G
	PA6	IO	SDA
	PA7	IO	SCL
	
	PB0	O	C1
	PB1	O	E
	PB2	O	C
	PB3	O	C3
	PB4	O	DP
	PB5	O	D
	PB6	O	C5
	PB7	I	RESET
*/


#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

//#include "suart.h"
#include "i2c.h"

#define	REFS_ACCC			(0)							// AVCC
#define	REFS_AREF			(_BV(REFS0))				// AREF
#define	REFS_256_NODECUP	(_BV(REFS1))				// 2.56V
#define	REFS_256_DECUP		(_BV(REFS0) | _BV(REFS1))	// 2.56V AREFڑ

typedef	unsigned char	BYTE;
typedef	unsigned short	WORD;
typedef unsigned char	BOOL;


#define MAKEWORD(a, b)	((WORD)(((BYTE)(a)) | ((WORD)((BYTE)(b))) << 8))
#define LOBYTE(w)		((BYTE)(w))
#define HIBYTE(w)		((BYTE)(((WORD)(w) >> 8) & 0xFF))

#define	BIT_LED_A		MAKEWORD((1<<0), 0)	// A
#define	BIT_LED_B		MAKEWORD((1<<2), 0)	// B
#define	BIT_LED_C		MAKEWORD(0, (1<<2))	// C
#define	BIT_LED_D		MAKEWORD(0, (1<<5))	// D
#define	BIT_LED_E		MAKEWORD(0, (1<<1))	// E
#define	BIT_LED_F		MAKEWORD((1<<3), 0)	// F
#define	BIT_LED_G		MAKEWORD((1<<5), 0)	// G
#define	BIT_LED_DP		MAKEWORD(0, (1<<4))	// DP
#define	BIT_LED_C1		MAKEWORD(0, (1<<0))	// C1
#define	BIT_LED_C2		MAKEWORD((1<<1), 0)	// C2
#define	BIT_LED_C3		MAKEWORD(0, (1<<3))	// C3
#define	BIT_LED_C4		MAKEWORD((1<<4), 0)	// C4
#define	BIT_LED_C5		MAKEWORD(0, (1<<6))	// C5

#define	BIT_LED_MASK_SEG	( BIT_LED_A | BIT_LED_B | BIT_LED_C | BIT_LED_D \
							| BIT_LED_E | BIT_LED_F | BIT_LED_G \
							| BIT_LED_DP )
#define	BIT_LED_MASK_COM	( BIT_LED_C1 | BIT_LED_C2 | BIT_LED_C3 | BIT_LED_C4 | BIT_LED_C5 )
#define	BIT_LED_MASK_ALL	( BIT_LED_MASK_SEG | BIT_LED_MASK_COM )


#define	ADDR_DS1720	(0x90 | (3 << 1))
#define	ADDR		ADDR_DS1720
#define	MAX_DIGITS	5

volatile static BYTE	g_byDigit[MAX_DIGITS];
volatile static BYTE	g_byDP;


WORD __attribute__ ((progmem))	g_segTbl[]	= {
	BIT_LED_A | BIT_LED_B | BIT_LED_C | BIT_LED_D | BIT_LED_E | BIT_LED_F,				// 0
	BIT_LED_B | BIT_LED_C,																// 1
	BIT_LED_A | BIT_LED_B | BIT_LED_D | BIT_LED_E | BIT_LED_G,							// 2
	BIT_LED_A | BIT_LED_B | BIT_LED_C | BIT_LED_D | BIT_LED_G,							// 3
	BIT_LED_B | BIT_LED_C | BIT_LED_F | BIT_LED_G,										// 4
	BIT_LED_A | BIT_LED_C | BIT_LED_D | BIT_LED_F | BIT_LED_G,							// 5
	BIT_LED_A | BIT_LED_C | BIT_LED_D | BIT_LED_E | BIT_LED_F | BIT_LED_G,				// 6
	BIT_LED_A | BIT_LED_B | BIT_LED_C | BIT_LED_F,										// 7
	BIT_LED_A | BIT_LED_B | BIT_LED_C | BIT_LED_D | BIT_LED_E | BIT_LED_F | BIT_LED_G,	// 8
	BIT_LED_A | BIT_LED_B | BIT_LED_C | BIT_LED_D | BIT_LED_F | BIT_LED_G,				// 9
};

WORD __attribute__ ((progmem))	g_comTbl[]	= {
	~BIT_LED_C1 & BIT_LED_MASK_COM,
	~BIT_LED_C2 & BIT_LED_MASK_COM,
	~BIT_LED_C3 & BIT_LED_MASK_COM,
	~BIT_LED_C4 & BIT_LED_MASK_COM,
	~BIT_LED_C5 & BIT_LED_MASK_COM,
};

/*
unsigned short adconv(unsigned char mux)
{
	ADMUX	= mux | REFS_256_DECUP | _BV(ADLAR);
	cbi(ADCSR, ADIF);
	sbi(ADCSR, ADSC);
	loop_until_bit_is_set(ADCSR, ADIF);
	return	inw(ADCL);
}
*/

static void ioInit()
{
	DDRA	= LOBYTE(BIT_LED_MASK_ALL);
	DDRB	= HIBYTE(BIT_LED_MASK_ALL);
	PORTA	= LOBYTE(BIT_LED_MASK_ALL);
	PORTB	= HIBYTE(BIT_LED_MASK_ALL);

/*
	sbi(PLLCSR, PLLE);								// Enable PLL
	loop_until_bit_is_set(PLLCSR, PLOCK);			// Wait for PLL to lock
	sbi(PLLCSR, PCKE);
	
	TCCR1A	= _BV(COM1A0) | _BV(PWM1A);				// Set PWM mode: toggle OC1A on compare
	OCR1C	= 0xff;
	OCR1A	= 0x00;
	TCCR1B	= _BV(CS11) | _BV(CS10) | _BV(CTC1);	// Enable Timer/Set PWM clock prescaler to PCK/4 (16MHz PWM clock)
*/

	TCCR0	= _BV(PSR0) | 0x02;						// CK/2
	ADCSR	= _BV(ADEN) | 0x03;						// CK/128

	i2c_master_init();

	sbi(TIMSK, TOIE0);
}


int main(void)
{
//	unsigned int	tmp	= 0;

	ioInit();

	// DS1721EϊJn
	i2c_master_start_condition();
	i2c_master_write_byte(ADDR);
	i2c_master_write_byte(0xAC);

	i2c_master_start_condition();
	i2c_master_write_byte(ADDR);
	i2c_master_write_byte((1 << 2));

	i2c_master_start_condition();
	i2c_master_write_byte(ADDR);
	i2c_master_write_byte(0x51);		// Start convert T (K{)

	g_byDigit[0]	= 0;
	g_byDigit[1]	= 0;
	g_byDigit[2]	= 0;
	g_byDigit[3]	= 0;
	g_byDigit[4]	= 0;
	g_byDP			= -1;

	sei();

	while(1)
	{
		volatile int	wait;
		signed char		th;
		unsigned char	tl;

		// x擾
		i2c_master_start_condition();
		i2c_master_write_byte(ADDR);
		i2c_master_write_byte(0xAA);
		i2c_master_start_condition();
		i2c_master_write_byte(ADDR | 0x01);
		th	= i2c_master_read_byte(1);
		tl	= i2c_master_read_byte(0);
		i2c_master_stop_condition();

		g_byDigit[0]	= th / 100;
		g_byDigit[1]	= (th / 10) % 100;
		g_byDigit[2]	= th % 10;
		g_byDigit[3]	= tl / 100;
		g_byDigit[4]	= (tl / 10) % 10;
		g_byDP			= 2;

//		for(wait = 0; wait < 1000; wait++);

//		tmp++;
	}

	return	0;
}



SIGNAL(SIG_OVERFLOW0)
{
	WORD		wTmp;
	static BYTE	cnt		= 0;

	if(cnt >= MAX_DIGITS)
		cnt	= 0;

	wTmp	= MAKEWORD(PORTA, PORTB);
	wTmp	&= ~BIT_LED_MASK_ALL;
	wTmp	|= pgm_read_word(&g_segTbl[g_byDigit[cnt]]);
	wTmp	|= pgm_read_word(&g_comTbl[cnt]);

	if(g_byDP == cnt)
		wTmp	|= BIT_LED_DP;

	PORTA	= LOBYTE(wTmp);
	PORTB	= HIBYTE(wTmp);

	cnt++;
}


